; ATTiny45 controlling a set of RGB LED's. 
; Example for creating a software/interrupt driven 3 channel PWM
;
; Floery Tobias - theFloe@tobiscorner.shellmaster.de
; www.tobiscorner.at.tf - the electronix Portal
;
;Adapted to ATTiny45 and minor changes by R.I.

.include "tn45def.inc"

.EQU	MODE0	= 0		; at Cold-Start only: Add Blue
.EQU	MODE1	= 1		; add Red
.EQU	MODE2	= 2		; remove Blue,leave Red only
.EQU	MODE3	= 3		; add Green
.EQU	MODE4	= 4		; remove Red,leave Green only
.EQU	MODE5	= 5		; add Blue
.EQU	MODE6 	= 6		; add Red
.EQU	MODE7	= 7		; remove Red and Green,leave Blue only,
						; then loop back to MODE1

.EQU	PWMPORT	= PORTB
.EQU	PWMDDR	= DDRB
.EQU	PPWM_B	= 0		; B(lue)  = PB0
.EQU	PPWM_R	= 1		; R(ed)   = PB1
.EQU	PPWM_G	= 2		; G(reen) = PB2

.EQU	PWM_MAX  = 0xFF	; Max PWM value to count to
;***************************************************
;    Things you can change to adapt to your need
;***************************************************
.EQU	TIMER0   = 0xA0 ; Speed of Color-Change.Influences
						; also the effect of the next two
						; Variables.  
.EQU	DIMMSTEP = 1	; Steps between each dimm interval 
.EQU	ENDDELAY = 0x80	; Waiting time after full color reached
;***************************************************


.DEF	t_sreg		=R1
.DEF	tmp			=R16
.DEF	pwm_b		=R17
.DEF	pwm_r		=R18
.DEF	pwm_g		=R19
.DEF	loopcnt		=R20
.DEF	mode		=R21
.DEF	dodelay		=R22



.CSEG
.ORG 0x0000			;Start of Interrupt-Vectors
	rjmp INIT		;Reset Handler
	reti			;INT0 (External Interrupt Request_0) Handler
	reti			;PCINT0 (Pin Change Interrupt Request_0) Handler
	reti			;Timer1_Comp_A (Timer1 Compare Match A) Handler
	rjmp TIMER1_ISR	;Timer1_OVF (Timer1 Overflow) Handler
	rjmp TIMER0_ISR	;Timer0_OVF (Timer0 Overflow) handler
	reti			;EE_RDY (EEPROM Ready) Handler
	reti			;ANA_Comp (Analog Comparator) Handler
	reti			;ADC (ADC Conversion complete) Handler 
	reti			;Timer1_Comp_B (Timer1 Compare Match B) Handler
	reti			;Timer0_Comp_A (Timer0 Compare Match A) Handler
	reti			;Timer0_Comp_B (Timer0 Compare Match B) Handler
	reti			;WDT (Watchdog Time-out) Handler
	reti			;USI_Start (USI Start) Handler
	reti			;USI_OVF (USI Overflow) Handler

;**************************************
; Initialize first
;**************************************		
INIT:
	ldi	r16,low(RAMEND) ;Init Stack-Pointer for ATtiny45
	ldi	r17,high(RAMEND)
	out	SPL,r16	
	out	SPH,r17

;INIT_PORTS:
	ldi	tmp, (1<<PPWM_B)|(1<<PPWM_R)|(1<<PPWM_G)
	out	PWMDDR, tmp		; define Pins as Output
	clr	tmp
	out	PWMPORT, tmp	; reset Output-Pins to logic "0"
						; Output-Pins are activ "1" !
;INIT_TIMER0:
	ldi	tmp, (1<<CS00)|(1<<CS02) ; Prescaler CK/256
	out	TCCR0B, tmp
	ldi	tmp, TIMER0			; preload Timer0
	out	TCNT0, tmp

;INIT_TIMER1:
	ldi	tmp, (1<<CS11)		;Prescaler CK/2
	out	TCCR1, tmp
	
;INIT_REGISTERS:
	clr	pwm_b
	clr	pwm_r
	clr	pwm_g
	clr	dodelay
	ldi	mode, MODE0

;ENABLE_TIMER_INTERRUPTS:
	ldi	tmp, (1<<TOIE1)|(1<<TOIE0)
	out	TIMSK, tmp
	sei
;******************************
;  "Main"-Program
;******************************
MAIN:				;wait for Interrupt
	nop					
	rjmp Main


;*************************************************
; Timer0 Interrupt Service Routine
; here the different color patterns are generated
;*************************************************

TIMER0_ISR:	
	in	t_sreg, SREG		; save SREG	(only for security)					
	ldi	tmp, TIMER0			; reload Timer1
	out	TCNT0, tmp
	cpi	dodelay, 0x00		; do i have to delay?
	breq	next0			; if no, change color
delay:						; do delay
	dec dodelay
	rjmp	timer0_exit		
next0:
	ldi	tmp, DIMMSTEP		; load dimming step
	cpi	mode, MODE0			; is mode0 active?
	brne	next1
	add	pwm_b, tmp			; yes,add more Blue
	brcc	timer0_exit		; color full?
	ldi	mode, MODE1			; set next mode
	ldi	pwm_b, 0xFF			; set color full
	ldi	dodelay, ENDDELAY	; set delay
	rjmp	timer0_exit
next1:						
	cpi	mode, MODE1			; is mode1 active?
	brne	next2			; no,check next Mode
	add	pwm_r, tmp			; yes,add more Red to Blue
	brcc	timer0_exit
	ldi	mode, MODE2
	ldi	pwm_r, 0xFF
	ldi	dodelay, ENDDELAY
	rjmp	timer0_exit
next2:
	cpi	mode, MODE2
	brne	next3
	sub	pwm_b, tmp			; remove Blue from Red
	brcc	timer0_exit
	ldi	mode, MODE3
	ldi	pwm_b, 0x00
	ldi	dodelay, ENDDELAY
	rjmp	timer0_exit
next3:
	cpi	mode, MODE3
	brne	next4
	add	pwm_g, tmp			; add more Green to Red
	brcc	timer0_exit
	ldi	mode, MODE4
	ldi	pwm_g,0xFF
	ldi	dodelay, ENDDELAY
	rjmp	timer0_exit
next4:
	cpi	mode, MODE4
	brne	next5
	sub	pwm_r, tmp			; remove Red,leave Green
	brcc	timer0_exit
	ldi	mode, MODE5
	ldi	pwm_r, 0x00
	ldi	dodelay, ENDDELAY
	rjmp	timer0_exit
next5:	
	cpi	mode, MODE5
	brne	next6
	add	pwm_b, tmp			; add more Blue to Green
	brcc	timer0_exit
	ldi	mode, MODE6
	ldi	pwm_b, 0xFF
	ldi	dodelay, ENDDELAY
	rjmp	timer0_exit
next6:
	cpi	mode, MODE6
	brne	next7
	add	pwm_r, tmp			; add more Red to Green and Blue
	brcc	timer0_exit
	ldi	mode, MODE7
	ldi	pwm_r, 0xFF
	ldi	dodelay, ENDDELAY
	rjmp	timer0_exit
next7:
	cpi	mode, MODE7
	brne	timer0_exit
	sub	pwm_g, tmp			; remove Green
	;sub	pwm_b, tmp
	sub	pwm_r, tmp			; remove Red
	brcc	timer0_exit
	ldi	pwm_b, 0xFF			; leave Blue only
	clr	pwm_r				
	clr	pwm_g
	ldi	mode, MODE1			; so we can start the cycle again...
	ldi	dodelay, ENDDELAY
timer0_exit:
	out SREG, t_sreg		; restore SREG
	reti 					; enable Interrupts and return
		
; Timer1 Interrupt Service Routine
; here the PWM is generated

TIMER1_ISR:
	in	t_sreg, SREG		; save SREG (only for security)
	dec	loopcnt				; decrement PWM cycle counter
	brne step0
	ldi	loopcnt, PWM_MAX	; one PWM-Cycle compleat,reload loopcnt
step0:
	cp pwm_b, loopcnt		; time to toggle Output?
	brcs step0a				; No
	sbi	PWMPORT, PPWM_B		; Yes
	rjmp	step1
step0a:
	cbi PWMPORT, PPWM_B
step1:
	cp pwm_r, loopcnt
	brcs step1a
	sbi	PWMPORT, PPWM_R
	rjmp	step2
step1a:
	cbi PWMPORT, PPWM_R
step2:
	cp pwm_g, loopcnt
	brcs step2a
	sbi	PWMPORT, PPWM_G
	rjmp	Exit_Timer1
step2a:
	cbi PWMPORT, PPWM_G
Exit_Timer1:
	out SREG, t_sreg				; restore SREG
	reti							; return from Interrupt
	
