; ATTiny15L controlling a set of RGB LED's. 
; Example for creating a software/interrupt driven 3 channel PWM
; Analog Input for Color - mixing
; Floery Tobias - theFloe@tobiscorner.shellmaster.de
; www.tobiscorner.at.tf - the electronix Portal

.include "tn15def.inc"

.EQU	MODE0	= 0
.EQU	MODE1	= 1
.EQU	MODE2	= 2
.EQU	MODE3	= 3
.EQU	MODE4	= 4
.EQU	MODE5	= 5
.EQU	MODE6 	= 6
.EQU	MODE7	= 7

.EQU	PWMPORT	= PORTB
.EQU	PWMDDR	= DDRB
.EQU	PPWM0	= 0		; B(lue) - connect to external FET or single LED
.EQU	PPWM1	= 4		; R(ed) - connect to external FET or single LED
.EQU	PPWM2	= 3		; G(reen) - connect to external FET or single LED

.EQU	PWM_MAX = 0xFF	; Max value to count to
.EQU	ENDDELAY = 0xFF	; Timout at end-state-reached

.DEF	tmp			=R16
.DEF	pwm_0		=R17
.DEF	pwm_1		=R18
.DEF	pwm_2		=R19
.DEF	loopcnt		=R20
.DEF	mode		=R21
.DEF	dodelay		=R22
.DEF	t_sreg		=R23


.CSEG
	rjmp INIT				; Reset Handle
	reti					; INT0
	reti					; PinChange
	reti					; Timer1 compare match
	rjmp TIMER1_ISR			; Overflow 1
	rjmp TIMER_ISR			; Overflow 0
	reti					; EEProm Ready
	reti					; Analog Comparator
	reti

; Timer0 Interrupt Service Routine
; here the different color patterns are generated

TIMER_ISR:	
	cli

	tst		dodelay					; do i have to delay?
	breq	next0					; if no, change color
delay:								; delay
	dec dodelay
	rjmp	next8	
		
next0:
	cpi	mode, MODE0
	brne	next1

	inc		pwm_0			; add blue

	cpi		pwm_0, PWM_MAX	; if full goto next color
	brne	next8	
	ldi		mode, MODE1	
	ldi	dodelay, ENDDELAY	; set delay
	rjmp	next8

next1:
	cpi	mode, MODE1
	brne	next2

	inc		pwm_1			; add red
	cpi		pwm_1, PWM_MAX
	brne	next8
	ldi	mode, MODE2
	ldi	dodelay, ENDDELAY
	rjmp	next8

next2:
	cpi	mode, MODE2
	brne	next3

	dec		pwm_0			; remove blue
	tst		pwm_0
	brne	next8
	ldi	mode, MODE3
	ldi	dodelay, ENDDELAY
	rjmp	next8

next3:
	cpi	mode, MODE3
	brne	next4

	inc		pwm_2			; add green
	cpi		pwm_2, PWM_MAX
	brne	next8
	ldi	mode, MODE4
	ldi	dodelay, ENDDELAY
	rjmp	next8

next4:
	cpi	mode, MODE4
	brne	next5

	dec		pwm_1			; remove red
	tst		pwm_1
	brne	next8
	ldi	mode, MODE5
	ldi	dodelay, ENDDELAY
	rjmp	next8

next5:
	cpi	mode, MODE5
	brne	next6

	inc		pwm_0			; add blue
	cpi		pwm_0, PWM_MAX
	brne	next8
	ldi	mode, MODE6
	ldi	dodelay, ENDDELAY
	rjmp	next8

next6:
	cpi	mode, MODE6
	brne	next7

	inc		pwm_1			; add red
	cpi		pwm_1, PWM_MAX
	brne	next8
	ldi	mode, MODE7
	ldi	dodelay, ENDDELAY
	rjmp	next8

next7:
	cpi	mode, MODE7
	brne	next8

	dec		pwm_1			; remove red
	dec		pwm_2			; remove green

	tst		pwm_1
	brne	next8
	ldi	mode, MODE1			; start again with blue
	ldi	dodelay, ENDDELAY
	
next8:
	sei						; enable Interrupts
	reti


; Timer1 Interrupt Service Routine
; here the PWM is generated
TIMER1_ISR:
	in	t_sreg, SREG

	dec	loopcnt				; decrement PWM cycle counter
	brne step0
	ldi	loopcnt, PWM_MAX
	
step0:
	cp pwm_0, loopcnt
	brcs step0a
	sbi	PWMPORT, PPWM0
	rjmp	step1
step0a:
	cbi PWMPORT, PPWM0

step1:
	cp pwm_1, loopcnt
	brcs step1a
	sbi	PWMPORT, PPWM1
	rjmp	step2
step1a:
	cbi PWMPORT, PPWM1

step2:
	cp pwm_2, loopcnt
	brcs step2a
	sbi	PWMPORT, PPWM2
	rjmp	step3
step2a:
	cbi PWMPORT, PPWM2
step3:

	out SREG, t_sreg
	reti							; return from Interrupt
	
INIT:

INIT_PORTS:
	ldi	tmp, (1<<PPWM0)|(1<<PPWM1)|(1<<PPWM2)
	out	PWMDDR, tmp
	clr	tmp
	out	PWMPORT, tmp
INIT_TIMER:
	ldi	tmp, 0x04	; Prescaler CK/256
	out	TCCR0, tmp

INIT_TIMER1:
	ldi	tmp, 0x02	; Prescaler CK*8
	out	TCCR1, tmp
	
MAIN:
	ldi	pwm_0, 0x00
	ldi	pwm_1, 0x00
	ldi	pwm_2, 0x00
	ldi	mode, MODE0
	
	clr	dodelay
	
	ldi	tmp, (1<<TOIE1)|(1<<TOIE0)
	out	TIMSK, tmp

	sei

ELOOP:
	nop
	rjmp ELOOP
